using Microsoft.Extensions.Logging;
using System.ComponentModel;
using System.Reflection;
using System.Windows.Forms;
using TransportCompanyContracts.BusinessLogicsContracts;
using TransportCompanyContracts.SearchModels;
using static System.Windows.Forms.VisualStyles.VisualStyleElement;

namespace TransportCompany
{
    public partial class FormTrucking : Form
    {
        private readonly ILogger _logger;

        private readonly ITruckingLogic _truckingLogic;

        private readonly IClientLogic _clientLogic;

        public FormTrucking(ILogger<FormTrucking> logger, ITruckingLogic truckingLogic, IClientLogic clientLogic)
        {
            InitializeComponent();

            _logger = logger;
            _truckingLogic = truckingLogic;
            _clientLogic = clientLogic;
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            LoadData();
        }

        private void LoadData()
        {
            _logger.LogInformation(" ");

            try
            {
                var list = _truckingLogic.ReadList(null);

                var listClients = _clientLogic.ReadList(null);

                if (list != null)
                {
                    dataGridView.DataSource = list;
                    dataGridView.Columns["ClientId"].Visible = false;
                    dataGridView.Columns["CargoId"].Visible = false;
                    dataGridView.Columns["TransportId"].Visible = false;
                    dataGridView.Columns["TransportationId"].Visible = false;
                }

                if (listClients != null)
                {
                    comboBoxEmails.DisplayMember = "Email";
                    comboBoxEmails.ValueMember = "Id";
                    comboBoxEmails.DataSource = listClients;
                    comboBoxEmails.SelectedItem = null;
                }

                _logger.LogInformation(" ");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "  ");
                MessageBox.Show(ex.Message, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCreateTrucking_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormCreateTrucking));

            if (service is FormCreateTrucking form)
            {
                form.ShowDialog();
                LoadData();
            }
        }

        private void TransportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormTransport));

            if (service is FormTransport form)
            {
                form.ShowDialog();
                LoadData();
            }
        }

        private void CargoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormCargo));

            if (service is FormCargo form)
            {
                form.ShowDialog();
                LoadData();
            }
        }

        private void ClientToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormClients));

            if (service is FormClients form)
            {
                form.ShowDialog();
                LoadData();
            }
        }

        private void TypeTransportationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormTypeTransportation));

            if (service is FormTypeTransportation form)
            {
                form.ShowDialog();
                LoadData();
            }
        }

        private void GenerationClientsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormRandomCreateClient));

            if (service is FormRandomCreateClient form)
            {
                form.ShowDialog();
                LoadData();
            }
        }

        private void GenerationTruckingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormRandomCreateTrucking));

            if (service is FormRandomCreateTrucking form)
            {
                form.ShowDialog();
                LoadData();
            }
        }

        private void ButtonUpdate_Click(object sender, EventArgs e)
        {
            LoadData();
        }

        private void ComboBoxEmails_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!checkBoxForFilterMode.Checked)
            {
                //dataGridView.DataSource = _truckingLogic.ReadList(null);
                LoadData();
            }
            else
            {
                dataGridView.DataSource = _truckingLogic.ReadList(null).Where(x => x.ClientId == comboBoxEmails.SelectedIndex).ToList();
            }
        }

        private void CheckBoxSorted_CheckedChanged(object sender, EventArgs e)
        {
            dataGridView.DataSource = _truckingLogic.ReadList(null).OrderByDescending(x => x.Price).ToList();
        }

        private void TestTimeGetDataToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormTimeCheck));

            if (service is FormTimeCheck form)
            {
                form.ShowDialog();
                LoadData();
            }
        }
    }
}